/**************************************************************************//**
*   @file   mb_interrupts.c
*   @brief  Interrupts handling functions driver file.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: $WCREV$
******************************************************************************/

#include <stdio.h>
#include "mb_interrupts.h"

extern volatile int rxData;
extern volatile unsigned int i;
extern volatile int admp441RxData;
extern volatile char admp441RxEn;
extern volatile int ampFactor;

/******************************************************************************
* @brief UART Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void uartIntHandler(void)
{
	// If UART has triggered a RX DATA Interrupt
	// store the read value in rxData
	if(Xil_In32(UART_BASEADDR + 0x08) & 0x01)
	{
		// Save read data from UART
		rxData = Xil_In32(UART_BASEADDR);
	}
}

/******************************************************************************
* @brief Timer Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void admp441IntHandler(void)
{
	unsigned long dataMask = 0xFFFFFF00;

	// If data reception is activated
	if(admp441RxEn == 1)
	{
		// If the frame was captured on WS low
		if(Xil_In32(ADMP441_BASEADDR + 0x08) == 0x00)
		{
			// Read data
			admp441RxData = ((Xil_In32(ADMP441_BASEADDR + 0x04) << 1) & dataMask) * ampFactor;
			i++;
		}
	}
}

/******************************************************************************
* @brief Enable Interrupts.
*
* @param None.
*
* @return None.
******************************************************************************/
void enableInterrupts(void)
{
	// Register UART Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, UART_INTERRUPT, (XInterruptHandler)uartIntHandler, (void *) 0);

	// Register TIMER0 Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, ADMP441_INTERRUPT, (XInterruptHandler)admp441IntHandler, (void *) 0);

	// Enable Interrupt Controller
	XIntc_MasterEnable(INTC_BASEADDR);
	XIntc_EnableIntr(INTC_BASEADDR, UART_INTERRUPT_MASK | ADMP441_INTERRUPT_MASK);

	// Enable UART Interrupts
	Xil_Out32(UART_BASEADDR + 0x0C, (1 << 4));

	// Enable MicroBlaze Interrupts
	microblaze_enable_interrupts();
}
